/******************************************************************************

  CCommodity.h

  Stephen Freund
  freunds@cs.stanford.edu

  This class keeps information about a single commodity in the network.
  In addition to providing the source and sink for the given commodity,
  methods allow for the increasing and reducing of demands at both ends 
  during the algorithm.

  Each Commodity also has an ID number associated with it.  These should
  be assigned in the order in which the Commodities are stored in the
  Nodes and EdgeQueues.  Not only does this number identify a commodity,
  but it also reflects where in these arrays the commodity would be
  located.

******************************************************************************/

#ifndef _H_CCommodity
#define _H_CCommodity

#include <iostream.h>
#include "Parameter.h"

class CNode;

class CCommodity {
  private:
    CNode *itsSource;           /* Node for source              */
    CNode *itsSink;             /* Node for sink                */        
    double itsOriginalDemand;   /* Always the original value    */
    double itsDemand;           /* Will change during execution */ 
    int itsId;                  /* identifier / array index     */
		
  public:

    /* constructors / destructor */
    CCommodity();
    CCommodity(int Id, CNode *source, CNode *sink, double demand);

    /* Accessors */
    CNode *Source(void)		{ return itsSource; }
    CNode *Sink(void)		{ return itsSink;	}
    double Demand(void)		{ return itsDemand; }
    double OriginalDemand(void)	{ return itsOriginalDemand; }
    int Id(void)	        { return itsId; }
		
    /* operations of its demand */
    void IncreaseDemand(void)  { itsDemand += itsOriginalDemand * GetSigma(); }
    void HalveDemand(void)     { itsDemand = itsOriginalDemand; }
    void DivideDemand(double a)     { itsDemand /= a; }
    void SetDemand(double newDemand)  { itsDemand = newDemand; }

    /* print the commodity to a stream */
    friend ostream& operator<<(ostream&,const CCommodity&);
};

#endif
