/******************************************************************************

  CHeap.h

  Stephen Freund
  freunds@cs.stanford.edu

  Implements a priority queue of HeapItems.  Supports all of the basic
  queue functions, as well as a ChangeKey function which will change the
  key of the given commodity to the new value.  In order to avoid
  the linear search through the queue for a commodity's id number, a
  parallel array is kept by which we can index the current location
  of a commodity's entry in the heap's array.

******************************************************************************/

#ifndef _H_CHeap
#define _H_CHeap

#include <iostream.h>


typedef struct {
  double itsKey;
  int itsCommodity;
} HeapItemT;

class CCommodity;

class CHeap  {
 private:
  int itsNumItems;           /* # currently in queue */
  int itsMaxItems;           /* max number in queue  */
  HeapItemT *itsItems;       /* Actual items         */
  int *itsLocationsInHeap;   /* array of indexes into 
				itsItems for commodities */  
  
  void Heapify(int node);
  void Swap(int a, int b);	
  
 public:
  /* constructor / destructor */
  CHeap() { itsNumItems = 0; itsItems = NULL; }
  CHeap(int maxItems);
  ~CHeap() { delete itsItems; }
  
  /* heap operations */
  double GetMaxKey(void)  {  return(itsItems[0].itsKey);  }
  HeapItemT ExtractMax(void);
  void Insert(double key, int commodity);
  void Insert(HeapItemT &item) { Insert(item.itsKey, item.itsCommodity); }
  void ChangeKey(int commodity, double newKey);
  int NumElemsInHeap()	{ return itsNumItems; }
  
  /* I/O */
  friend ostream& operator<<(ostream& out,const CHeap& node);
  friend istream& operator>>(istream& in, CHeap& node);
};

#endif






