/******************************************************************************
 
  Parameter.h

  Stephen Freund
  freunds@cs.stanford.edu

  Functions to compute and get the current values for sigma, alpha,
  and epsilon.

  To use undirected graphs, uncomment the UNDIRECTED constant below.

  To use formatted input rather than matrices, comment the USE_MATRIX
  line below.  Assumes that graphs and commodities are symmetric in 
  in the Matrix input, but not in the formatted input.

  To increase the degree all nodes to atleast the average degree 
  before boosting, define BOOST_DEGREE below.

******************************************************************************/

#ifndef _H_Parameter
#define _H_Parameter

class CGraph;

/* use undirected graphs */
//#define UNDIRECTED

/* input is in matrix form */
#define USE_MATRIX

/* increase degrees to atleast the average degree */
//#define BOOST_DEGREE

/* don't move commodities below - epsilon / m * demand */
//#define LOW_CUTOFF

/* print the flow at the end */
#define PRINT_FLOW

/*
 * random constants needed throughout the program.
 */
#define kDelta		       0.00000001
#define MaxCommodities         500
#define MaxDegree              100
#define kMinDecreasePerPhase   0.03

double GetEpsilon(void);
double GetSigma(void);
double GetAlpha(void);

/*
 * recompute the constants now.
 */
void ComputeParameters(CGraph *theGraph);

/*
 * Print the formulae used for sigma, alpha, epsilon 
 */
void PrintParameterFunctions(void);

#endif
 







